/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ai_h__
#define __ai_h__

typedef enum   _AIMove AIMove;
typedef enum   _AIFight AIFight;
typedef enum   _AIRoundState AIRoundState;
typedef struct _AI AI;

#include "player.h"
#include "pathfind.h"

/* Some internal states for the AI */
enum _AIMove
{
  AIM_WAITING = 0,
  AIM_ACCELERATING,
  AIM_MOVINGTO,
  AIM_RUNAWAY,
  AIM_BREAKING,
};

enum _AIFight
{
  AIF_WAITING = 0,
  AIF_TARGETING,
  AIF_FIREING,
};

enum _AIRoundState
{
  AIR_WAITING=0,
  AIR_MOVING,
  AIR_TURNING,
  AIR_TARGETING,
  AIR_SHOOTING,
};

struct _AI
{
  Player *player;

  /* How long the AI has spent in this turn */
  float turn, turn_time;

  /* Whether the AI has assessed the current situation */
  int situation_assessed;

  /* The path the AI is moving along */
  pathnode *path;

  /* Current target (if any) for weapons */
  Player *target;

  /* BOB: None of the below is used in the round based AI */

  /* current target for moves */
  float dest[3];
  /* internal move action */
  AIMove move;
  /* internal fight action */
  AIFight fight;
  /* a weapon is coming */
  int incoming;
  /* time from last fire */
  float last_fire;
  /* round-based state */
  AIRoundState round_state;
  /* last used distance */
  float last_dist;
};

/* Make a new AI */
AI *make_ai(Player *player);

/* Return TRUE if the player is AI controlled */
int is_ai_controlled(Player *player);

/* Update the players controlled by an AI */
void update_ai_players(World *world, float dt);

/* Configure the AI players */
void player_config_ai(World *world);

/* Get an AI player to select its name, tank etc */
void config_ai_player(Player *player);

/* Get an AI to buy weapons */
void ai_buy_weapons(Player *player);

/* Get an AI to buy items */
void ai_buy_items(Player *player);

/* configure AI weapons */
void player_shop_ai(World *world);

/* Update AI player for round-based game */
void update_ai_player_round(Player *player, float dt);

/* Draw what the AI is thinking */
void draw_ai(AI *ai);

/* Free an AI */
void free_ai(AI *ai);

#endif /* __ai_h__ */
