/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __bird_h__
#define __bird_h__

typedef enum   _BirdType BirdType;
typedef struct _Bird Bird;

#include "round.h"

/* Behavior for birds */
enum _BirdType
{
  BIRD_LINE=0,
  BIRD_CIRCLE,
};

/* Birds */
struct _Bird
{
  World *world;
  Game *game;
  Round *round;

  float pos[3];  /* bird position */
  float dir[3];  /* bird direction */
  float dest[3]; /* bird current destination */
  BirdType type; /* the kind of bird */
  float speed;
  float wspeed;
  float angle;  /* current bird vertical rotation */

  Bird *next, *last;
};

/* init the bird system */
int init_birds(World *world);

/* free the bird system */
void free_birds(World *world);

/* update existing birds */
void update_birds(Round *round, float dt);

/* update a bird */
void update_bird(Bird *bird, float dt);

/* draw the birds */
void draw_birds(Round *round);

/* draw a bird */
void draw_bird(Bird *bird);

/* create the birds for the current map */
void create_birds(Round *round, int nb);

/* Remove all birds */
void clear_birds(Round *round);

#endif /* __bird_h__ */
