/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* config.h
   Manage .natow/ directory  */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
/* !?! in the man mkdir is in 'fcntl.h', but
   on my linux RH7.1 it is in 'sys/stat.h' !!! */
#include <sys/stat.h>

#include "config.h"
#include "text.h"

static char natow_directory[1024]={'\0'};

/* get the .natow/ dir and create it (if possible) */
void init_configdir(World *world)
{
  char *home;

  home = getenv("HOME");

  if (home == NULL)
    {
    Neprintf("Cannot find home directory.\n");
    Neprintf("NATOW configuration system will not work.\n");
    return;
    }

  sprintf(natow_directory, "%s/.natow", home);

  /* try to create it in all cases */
  mkdir(natow_directory, (mode_t)0700);
}

/* set the video ressouces file name */
int get_config_video(char *file)
{
  if (natow_directory[0] == '\0')
    return(0);
  sprintf(file, "%s/videorc", natow_directory);
  return(1);
}

/* set the key definition file name */
int get_config_key(char *file)
{
  if (natow_directory[0] == '\0')
    return(0);
  sprintf(file, "%s/keyrc", natow_directory);
  return(1);
}

/* (try to) open the video configuration file */
FILE *open_config_video()
{
  char buffer[1024]={'\0'};
  FILE *ftmp=NULL;

  if (!get_config_video(buffer))
    {
    return(NULL);
    }
  if ((ftmp = fopen(buffer, "r")) == NULL)
    {
    return(NULL);
    }
  return(ftmp);
}

/* (try to) open the key configuration file */
FILE *open_config_key()
{
  char buffer[1024]={'\0'};
  FILE *ftmp=NULL;

  if (!get_config_key(buffer))
    {
    return(NULL);
    }
  if ((ftmp = fopen(buffer, "r")) == NULL)
    {
    return(NULL);
    }
  return(ftmp);
}


/* (try to) create the video configuration file */
FILE *create_config_video()
{
  char buffer[1024]={'\0'};
  FILE *ftmp=NULL;

  if (!get_config_video(buffer))
    {
    return(NULL);
    }
  if ((ftmp = fopen(buffer, "w")) == NULL)
    {
    return(NULL);
    }
  return(ftmp);
}

/* (try to) create the key configuration file */
FILE *create_config_key()
{
  char buffer[1024]={'\0'};
  FILE *ftmp=NULL;

  if (!get_config_key(buffer))
    {
    return(NULL);
    }
  if ((ftmp = fopen(buffer, "w")) == NULL)
    {
    return(NULL);
    }
  return(ftmp);
}


