/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __game_h__
#define __game_h__

typedef enum   _GameMode GameMode;
typedef struct _Game Game;

enum _GameMode
{
  TURN_BASED = 0,
  REAL_TIME,
  SINGLE_PLAYER
};

#include "world.h"
#include "map.h"
#include "player_list.h"
#include "round.h"
#include "level_list.h"
#include "objects.h"

/* A structure for a game */
struct _Game
{
  /* The base structure */
  World *world;

  /* The level this is being played in */
  LevelEntry *level;
  Map *map;

  /* The generic objects on this map */
  int nobjects;
  Object *object_list;

  /* The mode the game is, for example real-time or turn-based */
  GameMode mode;

  /* Physical properties of the world/map */
  float gravity;
  float sea_level, sea_variations, sea_frequency;
  float effective_sea_level;

  /* Properties for this game */
  int starting_cash;
  float driving_time;

  /* The players in this game */
  PlayerList *players;

  /* What round this is up to */
  int round_num, max_rounds;
  Round *round;
};

/* Create a game */
int create_game(World *world);

/* Set the game mode, i.e. turn-based/real-time/single-player */
int set_game_mode(Game *game, GameMode mode);

/* Set the level used in this game */
int set_game_level(Game *game, LevelEntry *level);

/* Set the gravity for a game */
int set_game_gravity(Game *game, float gravity);

/* Set the driving time for a game */
int set_game_driving_time(Game *game, float driving_time);

/* Set the number of rounds in a game */
int set_game_rounds(Game *game, int nrounds);

/* Set the starting cash in a game */
int set_game_starting_cash(Game *game, int starting_cash);

/* Start a game */
int start_game(Game *game);

/* End a game */
int end_game(Game *game);

/* Comparison function to compare entries for sorting */
int compare_game_winners(PlayerEntry **entry1, PlayerEntry **entry2);

/* Remove everything from the game */
int finish_game(Game *game);

/* Free the game structure and all below it */
int free_game(Game *game);

#endif /* __game_h__ */
