/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __keymap_h__
#define __keymap_h__

/* Used for keys control */
typedef enum _KeyID
{
  KC_eng=0, /* Go into movement mode */
  KC_vp1,   /* u. speed + */
  KC_vp2,   /* u. speed ++ */
  KC_vm1,   /* u. speed - */
  KC_vm2,   /* u. speed -- */
  KC_br,    /* emergency break */
  KC_hp2,   /* u. heading ++ */
  KC_hp1,   /* u. heading + */
  KC_hm1,   /* u. heading - */
  KC_hm2,   /* u. heading -- */
  KC_mp,    /* muzzle + */
  KC_mm,    /* muzzle - */
  KC_tp,    /* trust + */
  KC_tm,    /* trust - */
  KC_rp,    /* remove projectile */
  KC_thp1,  /* turret heading + */
  KC_thp2,  /* turret heading ++ */
  KC_thm1,  /* turret heading - */
  KC_thm2,  /* turret heading -- */
  KC_tpm1,  /* turret pitch - */
  KC_tpm2,  /* turret pitch -- */
  KC_tpp1,  /* turret pitch + */
  KC_tpp2,  /* turret pitch ++ */
  KC_cyc,   /* next unit */
  KC_menu,  /* menu key */
  KC_conf,  /* configure keymap */
  KC_pause, /* pause game */
  KC_video, /* video configuration screen */
  KC_weapon,/* cycle between weapons */
  /* to be extended */
} KeyID;  /* just use as a index access for the next struct */

/* As tructure which associates descriptions with values */
typedef struct _KeyEntry
{
  char *name;         /* Small description of the action */
  char *description;  /* Full description of the action */
  int key;            /* The associated key */
} KeyEntry;

/* The number of possible keys control. enough ? :o) */
#define KEYS_NUMBER 256

/* Shortcut for key access */
#define KL(n)  (world->key_list[n].key)

#include "world.h"

/* Get the keymap */
KeyEntry *get_keymap(void);

/* Find a key by name */
KeyEntry *find_key(const char *name);

/* Convert a keycode to a key name */
char *convert_key_to_name(int key);

/* Convert a key name to a key code */
int convert_name_to_key(const char *name);

/* Key-map list saving. just use to generate an initial key-map */
int save_keylist(World *world);

/* Read a key-map */
int read_keylist(World *world);

/* Manage key configuration */
void manage_edit_keymap_input(World *world, int key);

/* Draw keymap config screen */
void draw_keymap(World *world, int in_selected);

#endif /* __keymap_h__ */
