/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* lakes.c
   Deal with lakes on the map */

#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>

#include "lakes.h"
#include "tools.h"

/* this macro allocs an array */
#define ALLOC_ARRAY(tx, ty, type, ptr)   \
{ \
int i; \
ptr = (type**)malloc(sizeof(type*)*(tx)); \
for(i=0; i<(tx); i++) \
  ptr[i] = (type*)malloc(sizeof(type)*(ty)); \
}

/* this macro freed an array */
#define FREE_ARRAY(tx, ptr) \
{ \
int i; \
for(i=0; i<(tx); i++) \
  Nfree(ptr[i]); \
Nfree(ptr); \
}


/* these elements are temporary arrays used by my functions */
int **sel=NULL;
int **order=NULL;
int listx[512], listy[512]; /* lakes borders are not suppose to
                               be longer than 512 pts */
int pliste=0;

/* structure of a linked list, to generate the contour */
typedef struct _El
{
  int x, y;
  struct _El *n1, *n2, *n3;
}El;
El ***chn=NULL;


/* create a list element */
El *create_el(int x, int y)
{
  El *tmp;

  tmp = malloc(sizeof(El));  /* check error... */
  tmp->x = x;
  tmp->y = y;
  tmp->n1 = tmp->n2 = tmp->n3 = NULL;

  return(tmp);
}

/* link 'el' and 'nxt' */
void link_el(El*el, El*nxt)
{
  if (el->n1 == NULL)
    {
    el->n1 = (struct _El*)nxt;
    return;
    }
  if (el->n2 == NULL)
    {
    el->n2 = (struct _El*)nxt;
    return;
    }
  if (el->n3 == NULL)
    {
    el->n3 = (struct _El*)nxt;
    return;
    }
  fprintf(stderr, "Oooops! Internal bug!\n");
}

/* true if 'el' points on 'dest' */
int point_on(El * el, El * dest)
{
  if (el == NULL)
    return(0);
  if (dest == NULL)
    return(0);
  if ((el->n1 == dest)||(el->n2 == dest)||(el->n3 == dest))
    return(1);
  return(0);
}

/* remove all references on the given point */
void remove_ref(Map *map, El *el)
{
  int x, y;

  for(x=0; x<map->size_x; x++)
    for(y=0; y<map->size_y; y++)
      {
      if (sel[x][y] == 3)
        {
        if (chn[x][y]->n1 == el)
          {
          chn[x][y]->n1 = chn[x][y]->n2;
          chn[x][y]->n2 = chn[x][y]->n3;
          chn[x][y]->n3 = NULL;
          }
        else
        if (chn[x][y]->n2 == el)
          {
          chn[x][y]->n2 = chn[x][y]->n3;
          chn[x][y]->n3 = NULL;
          }
        else
        if (chn[x][y]->n3 == el)
          {
          chn[x][y]->n3 = NULL;
          }
        }
      }
}


int rd=1;
/* linkage recursif des elements */
void link_recurs(Map *map, El * el)
{
  int i=0, j=1, k, x, y;

  x = el->x;
  y = el->y;

  order[x][y] = rd++;

  for(k=3; k>=0; k--)
    {
    if (k == 0)
      {
      i = 1;
      j = 0;
      }
    else
    if (k == 1)
      {
      i = -1;
      j = 0;
      }
    else
    if (k == 2)
      {
      i = 0;
      j = -1;
      }
    else
    if (k == 3)
      {
      i = 0;
      j = 1;
      }
    if ((x+i>=0)&&(x+i<map->size_x)&&(y+j>=0)&&(y+j<map->size_y))
      if (sel[x+i][y+j] == 2)
        {/* lier */
        link_el(el, chn[x+i][y+j]);
        sel[x+i][y+j] = 3;
        link_recurs(map, chn[x+i][y+j]);
        }
    }
}


int compute_lake(Map *map, float ALT, int xx, int yy, int *dx, int *dy)
{
  int i=0, j=1, lx=0, ly=0, x, y, encore, nb, k;

  /* starting pt */
  sel[xx][yy] = 1;

  do
    {/* for each point */
    encore = 0;
    for(x=0; x<map->size_x; x++)
      for(y=0; y<map->size_y; y++)
        {
        if (sel[x][y] != 0) /* which is selected */
          {/* check neiboohord (I HATE this word!) */
          for(i=-1; i<=1; i++)
            for(j=-1; j<=1; j++)
              {/* check bounds */
              if ((x+i>=0)&&(x+i<map->size_x)&&(y+j>=0)&&(y+j<map->size_y)&&((i!=0)||(j!=0)))
                {/* if ok add it */
                if ((sel[x+i][y+j] == 0)&&(map->pos[x+i][y+j][1] < ALT))
                  {
                  encore = 1;
                  sel[x+i][y+j] = 1;
                  if ((x+i==0)||(x+i==map->size_x)||(y+j==0)||(y+j==map->size_y))
                    {
                    printf("The lake cross the map bounds. Aborted.\n");
                    return(0);
                    }
                  }
                }
              }
          }
        }
    }
  while(encore);


  /* compute borders */
  for(x=0; x<map->size_x; x++)
    for(y=0; y<map->size_y; y++)
      {
      if (sel[x][y] == 0) /* unselected point */
        {/* check for nei#!@$ */
        for(i=-1; i<=1; i++)
          for(j=-1; j<=1; j++)
            {/* ... */
            if ((x+i>=0)&&(x+i<map->size_x)&&(y+j>=0)&&(y+j<map->size_y)&&((i!=0)||(j!=0)))
              {/* at least a nei... inside -> ok */
              if (sel[x+i][y+j] == 1)
                {
                sel[x][y] = 2;
                lx = x;
                ly = y;  /* still used ? */
                }
              }
            }
        }
      }

  /* prepare the linked list */
  for(x=0; x<map->size_x; x++)
    for(y=0; y<map->size_y; y++)
      {
      if (sel[x][y] == 2)
        {
        chn[x][y] = create_el(x, y);
        }
      else
        {
        chn[x][y] = NULL;
        }
      }

  /* search a starting point (with two nei...) */
  for(x=0; x<map->size_x; x++)
    for(y=0; y<map->size_y; y++)
      {
      if (sel[x][y] == 2)
        {
        nb = 0;
        for(k=0; k<4; k++)
          {
          if (k == 0)
            {
            i = 1;
            j = 0;
            }
          else
          if (k == 1)
            {
            i = -1;
            j = 0;
            }
          else
          if (k == 2)
            {
            i = 0;
            j = -1;
            }
          else
          if (k == 3)
            {
            i = 0;
            j = 1;
            }
          if ((x+i>=0)&&(x+i<map->size_x)&&(y+j>=0)&&(y+j<map->size_y))
            {
            if (sel[x+i][y+j] == 2)
              nb++;
            }
          }
        if (nb == 2)
          {
          lx = x;
          ly = y;
          }
        }
      }
    
  /* create the list recursively */
  sel[lx][ly] = 3;
  *dx = lx;
  *dy = ly;
  link_recurs(map, chn[lx][ly]);

  /* just close the list */
  for(k=0; k<4; k++)
    {
    if (k == 0)
      {
      i = 1;
      j = 0;
      }
    else
    if (k == 1)
      {
      i = -1;
      j = 0;
      }
    else
    if (k == 2)
      {
      i = 0;
      j = -1;
      }
    else
    if (k == 3)
      {
      i = 0;
      j = 1;
      }
    if ((lx+i>=0)&&(lx+i<map->size_x)&&(ly+j>=0)&&(ly+j<map->size_y))
      {
      if ((sel[lx+i][ly+j] == 3)&&(!(point_on(chn[lx][ly], chn[lx+i][ly+j]))))
        {
        link_el(chn[lx+i][ly+j], chn[lx][ly]);
        break;
        }
      }
    }

  /* remove those without successor */
  do
    {
    encore = 0;
    for(x=0; x<map->size_x; x++)
      for(y=0; y<map->size_y; y++)
        {
        if (sel[x][y] == 3)
          {
          if (chn[x][y]->n1 == NULL)
            {
            sel[x][y] = 0;
            remove_ref(map, chn[x][y]);
            encore = 1;
            }
          }
        }
    }
  while(encore);

  return(1);
}

/* creates a lake structure. The lake includes (x,y) point
   and extend until altitude of (mx,my).  */
Lake *create_lake(Map *map, char *params)
{
  Lake *lk;
  int nbe, dx, dy, i, j;
  El *cur;
  int x, y, mx, my;

  if ((lk = malloc(sizeof(Lake))) == NULL)
    {
    fprintf(stderr, "Not enough memory. Lake creation aborted.\n");
    return(NULL);
    }

  /* x,y: starting point, mx, my: ending point */
  sscanf(params, "%d%d%d%d", &x, &y, &mx, &my);



  /* allocate all data */
  ALLOC_ARRAY(map->size_x, map->size_y, int, sel);
  ALLOC_ARRAY(map->size_x, map->size_y, int, order);
  ALLOC_ARRAY(map->size_x, map->size_y, El*, chn);
  for(i=0; i<map->size_x; i++)
    for(j=0; j<map->size_y; j++)
      {
      sel[i][j] = 0;
      order[i][j] = 0;
      chn[i][j] = NULL;
      }
  

  /* compute the lake */
  if (!compute_lake(map, map->pos[mx][my][1], x, y, &dx, &dy))
    {
    fprintf(stderr, "Problem while creating lake.\n");
    return(NULL);
    }

  /* create the point list */
  /* nb of elements */
  nbe = 0;
  cur = chn[dx][dy];
  do
    {
    nbe++;
    cur = cur->n1;
    }
  while(cur != chn[dx][dy]);
  /* prepare lists */
  lk->nb_c = nbe;
  lk->cx = malloc(sizeof(int)*nbe);
  lk->cy = malloc(sizeof(int)*nbe);
  cur = chn[dx][dy];
  nbe = 0;
  do
    {
    lk->cx[nbe] = cur->x;
    lk->cy[nbe] = cur->y;
    nbe++;
    cur = cur->n1;
    }
  while(cur != chn[dx][dy]);

  /* various attributes */
  lk->altitude = map->pos[mx][my][1];
  lk->drawing = 0;


  /* free data */
  FREE_ARRAY(map->size_x, sel);
  FREE_ARRAY(map->size_x, order);
  FREE_ARRAY(map->size_x, chn);


  return(lk);
}
