/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *                     Joel Pitt      <extropy@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __player_list_h__
#define __player_list_h__

typedef struct _PlayerList PlayerList;
typedef struct _PlayerEntry PlayerEntry;

#include "player.h"

/* A list of players */
struct _PlayerList
{
  int nplayers;

  PlayerEntry *first, *last;
};

/* One player from that list */
struct _PlayerEntry
{
  Player *player;

  PlayerEntry *next, *last;
};

/* Make a new player list */
PlayerList *new_player_list(void);

/* Add a player to the front of the player list */
int prepend_player(PlayerList *list, Player *player);

/* Add a player to the end of the player list */
int append_player(PlayerList *list, Player *player);

/* Sort the player list using built-in qsort */
int sort_player_list(PlayerList *list,
		     int (*cmp)(PlayerEntry **entry1, PlayerEntry **entry2));

/* Randomize the order of the players in the list */
int randomize_player_list(PlayerList *list);

/* Copy a player list */
PlayerList *copy_player_list(PlayerList *plist);

/* Remove all the players from the player list */
int clear_players(PlayerList *list);

/* Free a player list */
int free_player_list(PlayerList *plist);

#endif /* __player_list_h__ */
