/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __tank_h__
#define __tank_h__

typedef struct _Tank Tank;

#include <glass.h>
#include "world.h"

struct _Tank
{
  World *world;

  char *name, *comment;

#ifdef USE_GLASS
  /* The GLASS model for the tank */
  GlassLibrary *library;
  GlassObject *texture_model, *model;

  /* The GLASS variables */
  int heading, rot, rot_x, rot_y, rot_z, turret_heading, turret_pitch;

  /* The GLASS active points */
  int barrel;
#endif

  /* applied texture or 0 */
  GLuint texture;

  /* scales for the tank */
  float scalex, scaley, scalez;

  int has_engine;
  float mass, speed, acc, health;

  float underwater_ability, underwater_damage;

  Tank *last, *next;
};

/* Allocate space for a new tank */
Tank *allocate_tank(void);

/* Add a tank to the world */
void add_tank(World *world, Tank *tank);

/* Set a tanks name */
void set_tank_name(Tank *tank, const char *name);

/* Get a tanks name */
const char *get_tank_name(Tank *tank);

/* Set a tanks comment */
void set_tank_comment(Tank *tank, const char *comment);

/* Get a tanks comment */
const char *get_tank_comment(Tank *tank);

#ifdef USE_GLASS
/* Set the GLASS texture object used by the tank */
void set_tank_tex_model(Tank *tank, const char *fname);

/* Set the GLASS object used by the tank */
void set_tank_model(Tank *tank, const char *fname);

/* Get the GLASS object used by the tank */
GlassObject *get_tank_model(Tank *tank);
#endif

/* Set a tanks mass */
void set_tank_mass(Tank *tank, float mass);

/* Set a tanks maximum speed */
void set_tank_speed(Tank *tank, float speed);

/* Set tank texture */
void set_tank_texture(Tank *tank, char *texture);

/* Set tank scales */
void set_tank_scales(Tank *tank, char *scales);

/* Set a tanks maximum acceleration */
void set_tank_acc(Tank *tank, float acc);

/* Set a tanks health */
void set_tank_health(Tank *tank, float health);

/* Load tanks from a tank file, return FALSE if unable to open file */
int load_tanks(World *world, const char *fname);

/* Load a tank from a file */
Tank *load_tank(FILE *tank_file, const char *fname);

/* Find a tank by name, return NULL if not found  */
Tank *find_tank(World *world, const char *name);

/* Draw a tank */
void draw_tank(Tank *tank);

/* Free all the tanks in the world */
void free_tanks(World *world);

/* Free a tank */
void free_tank(Tank *tank);

#endif /* __tank_h__ */
